CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
75 lines
2.2 KiB
C++
75 lines
2.2 KiB
C++
// Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
// SPDX-License-Identifier: MIT
|
|
|
|
#pragma once
|
|
#include "ck/host_utility/hip_check_error.hpp"
|
|
|
|
namespace ck {
|
|
|
|
// Initialization flag of Barrier object, can be any value except for zero
|
|
static constexpr int BarrierInitFlag = 0x7856;
|
|
|
|
// 1) only the first thread-block in the synchronizaton group is supposed to call this function. It
|
|
// is the responsibility of the user to ensure the two integer values in p_control_bits are zeros
|
|
// before calling gms_init().
|
|
// 2) Aftercalling gms_reset(), the two integer values in p_control_bits will be zeros, so no
|
|
// repetitious initialization of p_control_bits buffer is required
|
|
static __device__ void gms_init(int NumWarps, int* p_control_bits)
|
|
{
|
|
union
|
|
{
|
|
int two32[2];
|
|
unsigned long one64;
|
|
} regs;
|
|
|
|
regs.two32[0] = BarrierInitFlag;
|
|
regs.two32[1] = NumWarps;
|
|
|
|
if(threadIdx.x == 0)
|
|
atomicCAS(reinterpret_cast<unsigned long*>(p_control_bits), 0, regs.one64);
|
|
};
|
|
|
|
// all the workgroups in the synchronization group is supposed to call this function
|
|
static __device__ void gms_barrier(int* p_control_bits)
|
|
{
|
|
constexpr int mask = WarpSize - 1;
|
|
|
|
if((threadIdx.x & mask) == 0)
|
|
{
|
|
// ensure the barrier object is initialized
|
|
do
|
|
{
|
|
const int r0 = __atomic_load_n(&p_control_bits[0], __ATOMIC_RELAXED);
|
|
|
|
if(r0 == BarrierInitFlag)
|
|
break;
|
|
|
|
} while(true);
|
|
|
|
// go ahead toward the barrier line
|
|
atomicSub(&p_control_bits[1], 1);
|
|
|
|
// wait until all warps have arrived
|
|
do
|
|
{
|
|
const int r1 = __atomic_load_n(&p_control_bits[1], __ATOMIC_RELAXED);
|
|
|
|
if(r1 == 0)
|
|
break;
|
|
|
|
} while(true);
|
|
};
|
|
};
|
|
|
|
// 1) Only the first thread-block in the synchronizaton group is supposed to call this function.
|
|
// 2) Aftercalling gms_reset(), the two integer values in p_control_bits will be zeros, so no
|
|
// repetitious initialization of p_control_bits buffer is required
|
|
static __device__ void gms_reset(int* p_control_bits)
|
|
{
|
|
// reset the barrier object
|
|
if(threadIdx.x == 0)
|
|
(void)atomicCAS(&p_control_bits[0], BarrierInitFlag, 0);
|
|
};
|
|
|
|
} // namespace ck
|